//
// Created by xiangning zhu on 2021/1/11.
//

#ifndef SAMI_CORE_DEF_H
#define SAMI_CORE_DEF_H

#ifdef __cplusplus
#define SAMI_CORE_EXTERN_C_BEGIN extern "C" {
#define SAMI_CORE_EXTERN_C_END   }
#else
#define SAMI_CORE_EXTERN_C_BEGIN
#define SAMI_CORE_EXTERN_C_END
#endif

#ifdef _WIN32
#ifdef SAMI_CORE_WIN_EXPORT
#define SAMI_CORE_DLL_EXPORT __declspec(dllexport)
#else
#define SAMI_CORE_DLL_EXPORT __declspec(dllimport)
#endif
#else
#define SAMI_CORE_DLL_EXPORT __attribute__((visibility("default")))
#endif
SAMI_CORE_EXTERN_C_BEGIN

/*************Base****************/
typedef enum SAMI_CORE_DLL_EXPORT contextType {
    SAMICoreContextType_Lic_Auth = 0,
    TokenVerifyOnlineContext,
    TokenVerifyOfflineContext,
} SAMICoreContextType;

struct LicAuthParams {
    bool offlineAuth;
    const unsigned char* licBuffer;
    int licBufferLen;
    const char* appId;
};

typedef struct TokenCreateParameter {
    const char* url;
    const char* accessKey;
    const char* secretKey;
    const char* appKey;
    const char* version;
    int expiration;
} SAMICoreTokenCreateParameter;

typedef struct TokenResult {
    const char* token;
    int expiration;
} SAMICoreTokenResult;

typedef struct TokenVerifyOnlineParameter {
    const char* url;
    const char* appKey;
    const char* token;
    const char* extra;  // json
} SAMICoreTokenVerifyOnlineParameter;

typedef struct TokenVerifyOfflineParameter {
    const char* appKey;
    const char* token;
    const char* extra;
} SAMICoreTokenVerifyOfflineParameter;

typedef enum SAMI_CORE_DLL_EXPORT identify {
    /**
     * SAMICoreIdentify_Processor_ContextWithString, used to create vocal preset from json string.
     * create handle with SAMICoreContextCreateParameter
     *
     * @see SAMICoreContextCreateParameter
     */
    SAMICoreIdentify_Processor_ContextWithString = 0,

    /**
     * SAMICoreIdentify_Processor_ContextWithFileName, used to create vocal preset from json file path.
     * create handle with SAMICoreContextCreateParameter
     *
     * @see SAMICoreContextCreateParameter
     */
    SAMICoreIdentify_Processor_ContextWithFileName,

    /**
     * SAMICoreIdentify_Processor, used to create processor by name,here is an example:
     * @code
     *  SAMICoreHandle handle;
     *  const char* name = "Gain";
     *  int ret = SAMICoreCreateHandleByIdentify(&handle, SAMICoreIdentify_Processor, name);
     *  assert(ret == SAMI_OK);
     * @endcode
     *
     * or, you can create processor by enum id, like this:
     * @code
     * int ret = SAMICoreCreateHandleByIdentify(&handle, SAMICoreIdentify_Processor_Compressor, NULL);
     * @endcode
     *
     * the currently supported processors' name are match to processor id,
     * here is a table shows the mapping between IDs and names
     *
     * +---------------------------------------+--------------+
     * | Id                                    | Name         |
     * +---------------------------------------+--------------+
     * | SAMICoreIdentify_Processor_Gain       | "Gain"       |
     * +---------------------------------------+--------------+
     * | SAMICoreIdentify_Processor_Compressor | "Compressor" |
     * +---------------------------------------+--------------+
     *
     *
     */
    SAMICoreIdentify_Processor = 100,

    /**
     * Name: "Gain"
     * Parameters:
     * +---------+-------------+-----------+-------------+------------+
     * | index   | name        | type      | range       | default    |
     * +---------+-------------+-----------+-------------+------------+
     * | 0       | "Gain dB"   | "Float"   | [-70, 35]   | 0          |
     * +---------+-------------+-----------+-------------+------------+
     */
    SAMICoreIdentify_Processor_Gain,

    /**
     * Name: "Compressor"
     * Parameters:
     * +---------+----------------------+-----------+-----------------+------------+
     *  | index   | name                 | type      | range           | default    |
     *  +---------+----------------------+-----------+-----------------+------------+
     *  | 0       | "Bypass"             | "Bool"    | [false, true]   | false      |
     *  +---------+----------------------+-----------+-----------------+------------+
     *  | 1       | "Ratio"              | "Float"   | [1, 40]         | 1          |
     *  +---------+----------------------+-----------+-----------------+------------+
     *  | 2       | "Threshold dB"       | "Float"   | [-70, 0]        | 0          |
     *  +---------+----------------------+-----------+-----------------+------------+
     *  | 3       | "Knee dB"            | "Float"   | [0, 18]         | 6          |
     *  +---------+----------------------+-----------+-----------------+------------+
     *  | 4       | "Attack"             | "Float"   | [0.005, 250]    | 0.25       |
     *  +---------+----------------------+-----------+-----------------+------------+
     *  | 5       | "Release"            | "Float"   | [1, 2500]       | 100        |
     *  +---------+----------------------+-----------+-----------------+------------+
     *  | 6       | "Auto Makeup Gain"   | "Bool"    | [false, true]   | true       |
     *  +---------+----------------------+-----------+-----------------+------------+
     *  | 7       | "Output Gain dB"     | "Float"   | [-35, 35]       | 0          |
     *  +---------+----------------------+-----------+-----------------+------------+
     *  | 8       | "Lookahead"          | "Float"   | [0, 10]         | 0          |
     *  +---------+----------------------+-----------+-----------------+------------+
     */
    SAMICoreIdentify_Processor_Compressor,

    /**
     * Name: "Chorus"
     * Parameters:
     * +---------+-------------------------+-----------+-----------------+------------+
     * | index   | name                    | type      | range           | default    |
     * +---------+-------------------------+-----------+-----------------+------------+
     * | 0       | "Bypass"                | "Bool"    | [false, true]   | false      |
     * +---------+-------------------------+-----------+-----------------+------------+
     * | 1       | "Delay"                 | "Float"   | [0.5, 20]       | 4          |
     * +---------+-------------------------+-----------+-----------------+------------+
     * | 2       | "Rate"                  | "Float"   | [0, 4]          | 1          |
     * +---------+-------------------------+-----------+-----------------+------------+
     * | 3       | "Depth"                 | "Float"   | [0, 5]          | 2          |
     * +---------+-------------------------+-----------+-----------------+------------+
     * | 4       | "Stereo Phase Offset"   | "Float"   | [0, 1]          | 1          |
     * +---------+-------------------------+-----------+-----------------+------------+
     * | 5       | "Feedback"              | "Float"   | [0, 0.96]       | 0          |
     * +---------+-------------------------+-----------+-----------------+------------+
     * | 6       | "Wetness"               | "Float"   | [0, 1]          | 1          |
     * +---------+-------------------------+-----------+-----------------+------------+
     */
    SAMICoreIdentify_Processor_Chorus,

    /**
     * Name: "Distortion"
     * Parameters:
     * +---------+--------------------------+------------+------------------------------------------------------------------------------------------------------------------------------------------------------------------+------------+
     * | index   | name                     | type       | range | default    |
     * +---------+--------------------------+------------+------------------------------------------------------------------------------------------------------------------------------------------------------------------+------------+
     * | 0       | "Bypass"                 | "Bool"     | [false, true] | false      |
     * +---------+--------------------------+------------+------------------------------------------------------------------------------------------------------------------------------------------------------------------+------------+
     * | 1       | "Distortion Mode"        | "Choice"   | ["Cubic", "Tanh", "Arctan", "ArctanTanh", "L1", "L2",
     * "HardClip", "DeadZone", "FoldBackTriangle", "FoldBackSine", "FoldBackSinArctan", "BitCrush", "Decimate"]   |
     * "Cubic"    |
     * +---------+--------------------------+------------+------------------------------------------------------------------------------------------------------------------------------------------------------------------+------------+
     * | 2       | "Distortion Amount"      | "Float"    | [0, 1] | 0          |
     * +---------+--------------------------+------------+------------------------------------------------------------------------------------------------------------------------------------------------------------------+------------+
     * | 3       | "Apply Upsampling"       | "Choice"   | ["None", "X8", "X16", "X32"] | "None"     |
     * +---------+--------------------------+------------+------------------------------------------------------------------------------------------------------------------------------------------------------------------+------------+
     * | 4       | "Wetness"                | "Float"    | [0, 1] | 1          |
     * +---------+--------------------------+------------+------------------------------------------------------------------------------------------------------------------------------------------------------------------+------------+
     * | 5       | "Post Distortion Gain"   | "Float"    | [-40, 0] | 0          |
     * +---------+--------------------------+------------+------------------------------------------------------------------------------------------------------------------------------------------------------------------+------------+
     */
    SAMICoreIdentify_Processor_Distortion,

    /**
     * Name: "Echo"
     * Parameters:
     * +---------+----------------------+-----------+-----------------+------------+
     * | index   | name                 | type      | range           | default    |
     * +---------+----------------------+-----------+-----------------+------------+
     * | 0       | "Dry Bypass"         | "Bool"    | [false, true]   | false      |
     * +---------+----------------------+-----------+-----------------+------------+
     * | 1       | "Dry Gain dB"        | "Float"   | [-35, 6]        | 0          |
     * +---------+----------------------+-----------+-----------------+------------+
     * | 2       | "Dry Pan"            | "Float"   | [-1, 1]         | 0          |
     * +---------+----------------------+-----------+-----------------+------------+
     * | 3       | "Tap 1 Bypass"       | "Bool"    | [false, true]   | true       |
     * +---------+----------------------+-----------+-----------------+------------+
     * | 4       | "Tap 1 Delay Time"   | "Float"   | [0.001, 1]      | 0.001      |
     * +---------+----------------------+-----------+-----------------+------------+
     * | 5       | "Tap 1 Feedback"     | "Float"   | [0, 1]          | 0          |
     * +---------+----------------------+-----------+-----------------+------------+
     * | 6       | "Tap 1 Gain dB"      | "Float"   | [-35, 6]        | 0          |
     * +---------+----------------------+-----------+-----------------+------------+
     * | 7       | "Tap 1 Pan"          | "Float"   | [-1, 1]         | 0          |
     * +---------+----------------------+-----------+-----------------+------------+
     * | 8       | "Tap 1 Sync"         | "Bool"    | [false, true]   | true       |
     * +---------+----------------------+-----------+-----------------+------------+
     * | 9       | "Tap 2 Bypass"       | "Bool"    | [false, true]   | true       |
     * +---------+----------------------+-----------+-----------------+------------+
     * | 10      | "Tap 2 Delay Time"   | "Float"   | [0.001, 1]      | 0.001      |
     * +---------+----------------------+-----------+-----------------+------------+
     * | 11      | "Tap 2 Feedback"     | "Float"   | [0, 1]          | 0          |
     * +---------+----------------------+-----------+-----------------+------------+
     * | 12      | "Tap 2 Gain dB"      | "Float"   | [-35, 6]        | 0          |
     * +---------+----------------------+-----------+-----------------+------------+
     * | 13      | "Tap 2 Pan"          | "Float"   | [-1, 1]         | 0          |
     * +---------+----------------------+-----------+-----------------+------------+
     * | 14      | "Tap 2 Sync"         | "Bool"    | [false, true]   | true       |
     * +---------+----------------------+-----------+-----------------+------------+
     */
    SAMICoreIdentify_Processor_Echo,

    /**
     * Name: "Filter"
     * Parameters:
     * +---------+-----------------+------------+---------------------------------------------------------------------------------------------------------------------------+--------------+
     * | index   | name            | type       | range | default      |
     * +---------+-----------------+------------+---------------------------------------------------------------------------------------------------------------------------+--------------+
     * | 0       | "Filter Mode"   | "Choice"   | ["LowPass", "HighPass", "BandPass", "NormalisedBandPass", "Bell",
     * "HighShelf", "LowShelf", "AllPass", "Notch", "Morph"]   | "LowPass"    |
     * +---------+-----------------+------------+---------------------------------------------------------------------------------------------------------------------------+--------------+
     * | 1       | "Cutoff Freq"   | "Float"    | [20, 20000] | 1000         |
     * +---------+-----------------+------------+---------------------------------------------------------------------------------------------------------------------------+--------------+
     * | 2       | "Resonance"     | "Float"    | [0.1, 20] | 1            |
     * +---------+-----------------+------------+---------------------------------------------------------------------------------------------------------------------------+--------------+
     * | 3       | "Gain dB"       | "Float"    | [-20, 20] | 0            |
     * +---------+-----------------+------------+---------------------------------------------------------------------------------------------------------------------------+--------------+
     * | 4       | "Morph"         | "Float"    | [-1, 1] | 0            |
     * +---------+-----------------+------------+---------------------------------------------------------------------------------------------------------------------------+--------------+
     */
    SAMICoreIdentify_Processor_Filter,

    /**
     * Name: "GainAndPan"
     * Parameters:
     * +---------+-------------+-----------+-------------+------------+
     * | index   | name        | type      | range       | default    |
     * +---------+-------------+-----------+-------------+------------+
     * | 0       | "Gain dB"   | "Float"   | [-70, 35]   | 0          |
     * +---------+-------------+-----------+-------------+------------+
     * | 1       | "Pan"       | "Float"   | [-1, 1]     | 0          |
     * +---------+-------------+-----------+-------------+------------+
     */
    SAMICoreIdentify_Processor_GainAndPan,

    /**
     * Name: "Limiter"
     * Parameters:
     * +---------+-------------------------+-----------+-----------------+------------+
     * | index   | name                    | type      | range           | default    |
     * +---------+-------------------------+-----------+-----------------+------------+
     * | 0       | "Bypass"                | "Bool"    | [false, true]   | false      |
     * +---------+-------------------------+-----------+-----------------+------------+
     * | 1       | "Input Gain dB"         | "Float"   | [0, 24]         | 0          |
     * +---------+-------------------------+-----------+-----------------+------------+
     * | 2       | "Ceiling dB"            | "Float"   | [-24, 0]        | -0.3       |
     * +---------+-------------------------+-----------+-----------------+------------+
     * | 3       | "Release"               | "Float"   | [1, 3000]       | 500        |
     * +---------+-------------------------+-----------+-----------------+------------+
     * | 4       | "Lookahead"             | "Float"   | [0, 10]         | 0          |
     * +---------+-------------------------+-----------+-----------------+------------+
     * | 5       | "Audition Unity Gain"   | "Bool"    | [false, true]   | false      |
     * +---------+-------------------------+-----------+-----------------+------------+
     */
    SAMICoreIdentify_Processor_Limiter,

    /**
     * Name: "Reverb"
     * Parameters:
     * +---------+----------------------+-----------+-----------------+------------+
     * | index   | name                 | type      | range           | default    |
     * +---------+----------------------+-----------+-----------------+------------+
     * | 0       | "Bypass"             | "Bool"    | [false, true]   | false      |
     * +---------+----------------------+-----------+-----------------+------------+
     * | 1       | "Decay Time"         | "Float"   | [0.5, 16]       | 3          |
     * +---------+----------------------+-----------+-----------------+------------+
     * | 2       | "Damp"               | "Float"   | [0, 1]          | 0.5        |
     * +---------+----------------------+-----------+-----------------+------------+
     * | 3       | "Mod Depth"          | "Float"   | [0, 1]          | 0          |
     * +---------+----------------------+-----------+-----------------+------------+
     * | 4       | "Mod Rate"           | "Float"   | [0.01, 7.5]     | 0          |
     * +---------+----------------------+-----------+-----------------+------------+
     * | 5       | "Wet Gain"           | "Float"   | [-36, 12]       | 0          |
     * +---------+----------------------+-----------+-----------------+------------+
     * | 6       | "Wet Stereo Width"   | "Float"   | [0, 2]          | 1          |
     * +---------+----------------------+-----------+-----------------+------------+
     * | 7       | "Wet Mix"            | "Float"   | [0, 1]          | 0.5        |
     * +---------+----------------------+-----------+-----------------+------------+
     * | 8       | "Wet LP Bypass"      | "Bool"    | [false, true]   | true       |
     * +---------+----------------------+-----------+-----------------+------------+
     * | 9       | "Wet HP Bypass"      | "Bool"    | [false, true]   | true       |
     * +---------+----------------------+-----------+-----------------+------------+
     * | 10      | "Wet LP Freq"        | "Float"   | [20, 20000]     | 20000      |
     * +---------+----------------------+-----------+-----------------+------------+
     * | 11      | "Wet HP Freq"        | "Float"   | [20, 20000]     | 20         |
     * +---------+----------------------+-----------+-----------------+------------+
     * | 12      | "Wet LP Res"         | "Float"   | [0.1, 20]       | 1          |
     * +---------+----------------------+-----------+-----------------+------------+
     * | 13      | "Wet HP Res"         | "Float"   | [0.1, 20]       | 1          |
     * +---------+----------------------+-----------+-----------------+------------+
     */
    SAMICoreIdentify_Processor_Reverb,

    /**
     * Name: "Vibrato"
     * Parameters:
     * +---------+--------------------+-----------+-----------------+------------+
     * | index   | name               | type      | range           | default    |
     * +---------+--------------------+-----------+-----------------+------------+
     * | 0       | "Bypass"           | "Bool"    | [false, true]   | false      |
     * +---------+--------------------+-----------+-----------------+------------+
     * | 1       | "Rate Hz"          | "Float"   | [1, 8]          | 4          |
     * +---------+--------------------+-----------+-----------------+------------+
     * | 2       | "Depth Semitone"   | "Float"   | [0, 2]          | 0.5        |
     * +---------+--------------------+-----------+-----------------+------------+
     * | 3       | "Wetness"          | "Float"   | [0, 1]          | 1          |
     * +---------+--------------------+-----------+-----------------+------------+
     */
    SAMICoreIdentify_Processor_Vibrato,

    /**
     * Name: "TimeDomainPitchShifter"
     * Parameters:
     * +---------+------------------------+------------+------------------------------------------------------------------------------+----------------+
     * | index   | name                   | type       | range                                                                        | default        |
     * +---------+------------------------+------------+------------------------------------------------------------------------------+----------------+
     * | 0       | "Bypass"               | "Bool"     | [false, true]                                                                | false          |
     * +---------+------------------------+------------+------------------------------------------------------------------------------+----------------+
     * | 1       | "Pitch Ratio"          | "Float"    | [0.25, 4]                                                                    | 1              |
     * +---------+------------------------+------------+------------------------------------------------------------------------------+----------------+
     * | 2       | "Interpolation Mode"   | "Choice"   | ["Linear", "Lagrange4", "Lagrange16", "Lagrange24", "Sinc32", "Lanczos32"]   | "Lagrange4"    |
     * +---------+------------------------+------------+------------------------------------------------------------------------------+----------------+
     * | 3       | "Mono Input"           | "Bool"     | [false, true]                                                                | true           |
     * +---------+------------------------+------------+------------------------------------------------------------------------------+----------------+
     * | 4       | "Mode"                 | "Choice"   | ["Fastest", "Moderate", "HighQuality", "UltraHighQuality"]                   | "Moderate"     |
     * +---------+------------------------+------------+------------------------------------------------------------------------------+----------------+
     */
    SAMICoreIdentify_Processor_TimeDomainPitchShifter,

    /**
     * Name: "SpatialAudio"
     * Parameters:
     * +---------+------------------------+------------+------------------------------------------------------------------------------+----------------+
     * | index   | name                   | type       | range                                                                        | default        |
     * +---------+------------------------+------------+------------------------------------------------------------------------------+----------------+
     * | 0       | "preset_id"            | "Int"      | [0, 1]                                                                       | 1              |
     * +---------+------------------------+------------+------------------------------------------------------------------------------+----------------+
     */
    SAMICoreIdentify_Processor_SpatialAudio,

    // extractor
    SAMICoreIdentify_Extractor_F0Detection = 200,
    SAMICoreIdentify_Extractor_OnsetDetection,
    SAMICoreIdentify_Extractor_SpectrumDisplay,
    SAMICoreIdentify_Extractor_VolumeDetection,
    SAMICoreIdentify_Extractor_EventDetection,
    SAMICoreIdentify_Extractor_VoiceActivityDetection,
    SAMICoreIdentify_Extractor_BeatTracking,
    SAMICoreIdentify_Extractor_BeatTrackingOffline,
    SAMICoreIdentify_Extractor_Music2VibesVideoModel,
    SAMICoreIdentify_Extractor_Music2VibesMatchModel,
    SAMICoreIdentify_Extractor_NNVAD,
    SAMICoreIdentify_Extractor_Loudness,

    // 3a
    SAMICoreIdentify_AEC = 300,
    SAMICoreIdentify_RNNAEC,
    SAMICoreIdentify_AGC,
    SAMICoreIdentify_RNNOSIE16K,
    SAMICoreIdentify_RNNOSIE,
    SAMICoreIdentify_RNNOSIE48K,
    SAMICoreIdentify_TCNDENOISE44K,
    SAMICoreIdentify_TCNDENOISE16K,
    SAMICoreIdentify_DENOISE_V2_MUSIC44k,
    SAMICoreIdentify_DENOISE_V2_SPEECH44k,
    SAMICoreIdentify_DENOISE_V2_MUSIC16k,
    SAMICoreIdentify_DENOISE_V2_SPEECH16k,

    // non realtime effect
    SAMICoreIdentify_TimeScaler = 400,

    // sami engine
    SAMICoreIdentify_Engine = 500,
    SAMICoreIdentify_EngineExecutor,
    SAMICoreIdentify_EngineExecutor_TimeAlign,
    SAMICoreIdentify_EngineExecutor_AED,
    SAMICoreIdentify_EngineExecutor_Denoise,

    // server
    SAMICoreIdentify_Create_Token = 600,
    SAMICoreIdentify_TTS_Online = 601,
    SAMICoreIdentify_ASR_Online = 602,
    SAMICoreIdentify_BeatTracking_Online = 603,
    SAMICoreIdentify_MusicSourceSeparate_Online = 604,
    SAMICoreIdentify_MusicTagging_Online = 605,
    SAMICoreIdentify_Chorus_Online = 606,
    SAMICoreIdentify_MusicRetarget_Online = 607,
    SAMICoreIdentify_Midi_Online = 608,

    //tob
    SAMICoreIdentify_Processor_ContextWithBinaryRes = 2000,

    SAMICoreIdentify_TuneToTarget = 3000,
} SAMICoreIdentify;

typedef enum SAMI_CORE_DLL_EXPORT propertyId {
    // base parameter
    SAMICorePropertyID_Base_NAME = 0,
    SAMICorePropertyID_Base_UNKNOWN,

    // context
    SAMICorePropertyID_Processor_ContextPrepare = 100,
    SAMICorePropertyID_JSON_Context,

    // vad parameters
    SAMICorePropertyID_VAD_Aggressiveness_Mode_Name = 200,
    SAMICorePropertyID_VAD_Silence_Robust_Length_name,
    SAMICorePropertyID_VAD_Frame_Time_Length_Name,
    SAMICorePropertyID_VAD_Smoothing_Searching_Width_Name,
    SAMICorePropertyID_VAD_Smoothing_Searching_Threshold_Ratio_Name,
    SAMICorePropertyID_VAD_Overall_Moving_Average_Width_Name,
    SAMICorePropertyID_VAD_Overall_Searching_Threshold_Ratio_Name,
    SAMICorePropertyID_VAD_Overall_Silence_Length_Name,

    // compressor
    SAMICorePropertyID_Compressor_Bypass = 300,
    SAMICorePropertyID_Compressor_Ratio,
    SAMICorePropertyID_Compressor_Threshold,
    SAMICorePropertyID_Compressor_Knee_dB,
    SAMICorePropertyID_Compressor_Attack,
    SAMICorePropertyID_Compressor_Release,
    SAMICorePropertyID_Compressor_Auto_Makeup_Gain,
    SAMICorePropertyID_Compressor_Output_Gain_dB,
    SAMICorePropertyID_Compressor_Look_Head,

    // extractor frame feature
    SAMICorePropertyID_FrameFeatures = 400,
    SAMICorePropertyID_FrameFeature_SPECTRUM,
    SAMICorePropertyID_FrameFeature_ONSET,
    SAMICorePropertyID_FrameFeature_F0,
    SAMICorePropertyID_FrameFeature_BEAT_TRACKING,
    SAMICorePropertyID_FrameFeature_VOLUME,
    SAMICorePropertyID_FrameFeature_NNVAD,
    SAMICorePropertyID_FrameFeature_Voice_Activity,

    // extractor overall feature
    SAMICorePropertyID_OverallFeatures = 500,
    SAMICorePropertyID_OverallFeature_GLOBAL_LOUDNESS,
    SAMICorePropertyID_OverallFeature_GLOBAL_PEAK,
    SAMICorePropertyID_OverallFeature_TRACKING_OFFLINE_OVERALL,
    SAMICorePropertyID_OverallFeature_NNVAD_OVERALL,
    SAMICorePropertyID_OverallFeature_ONSET_OVERALL,
    SAMICorePropertyID_OverallFeature_Voice_Activity_OVERALL,

    // extractor
    SAMICorePropertyID_SetExtractorFloatParameter = 600,
    SAMICorePropertyID_SetExtractorStringParameter,

    // model path
    SAMICorePropertyId_LoadModelBinary = 700,

    // seek
    SAMICorePropertyId_Seek = 800,

    // time scaler
    SAMICorePropertyId_TimeScaler_ScaleRate = 900,

    // processor
    SAMICorePropertyID_Processor_Prepare = 1000,
    SAMICorePropertyID_Processor_SetParametersOffline,
    SAMICorePropertyID_Processor_ContextUpdateFromFile,
    SAMICorePropertyID_Processor_ContextUpdateFromString,
    SAMICorePropertyID_Processor_ContextEmplaceParameterEventNow,

    // sami engine
    SAMICoreEngineExecutorOutPut = 1100,

    // denoise
    SAMICorePropertyID_TCNDenoise = 1200,
    SAMICorePropertyID_TCNDenoise_Rate,
    SAMICorePropertyID_TCNDenoise_Mode,

    SAMICorePropertyID_Processor_ContextChunkBinaryRes = 1300,

    // denoise
    SAMICorePropertyID_Denoise_V2 = 1400,
    SAMICorePropertyID_Denoise_V2_Rate,
    SAMICorePropertyID_Denoise_V2_Mode,
    // todo add parameters of other algorithms
} SAMICorePropertyId;

typedef enum SAMI_CORE_DLL_EXPORT logLevel {
    Debug = 4,
    Error = 16,
    Warning = 32,
    Info = 128,
} SAMICoreLogLevel;

typedef enum SAMI_CORE_DLL_EXPORT dataType {
    // base
    SAMICoreDataType_Float = 0,
    SAMICoreDataType_Int,
    SAMICoreDataType_String,

    // custom base
    SAMICoreDataType_AudioBuffer = 50,

    // context
    SAMICoreDataType_ContextPrepareParameter = 100,
    SAMICoreDataType_ResourceParameter,
    SAMICoreDataType_ParameterEvent,

    // processor
    SAMICoreDataType_ProcessorParameter = 150,

    // Extractor
    SAMICoreDataType_FeatureSet = 250,
    SAMICoreDataType_ExtractorParameter,

    // model binary
    SAMICoreDataType_ModelBin = 300,

    // aec
    SAMICoreDataType_AecInput = 350,

    // sami engine tts
    SAMICoreDataType_TTSCallBack = 400,
    SAMICoreDataType_EngineCreateParameter,
    SAMICoreDataType_ExecutorCreateParameter,

    // time align
    SAMICoreDataType_TimeAlign = 450,

    // server
    SAMICoreDataType_TTS_Result = 500,
    SAMICoreDataType_ASR_Result = 503,
    SAMICoreDataType_AudioBin = 504,
    SAMICoreDataType_BeatTracking_Result = 505,
    SAMICoreDataType_MusicSourceSeparate_Result = 506,
    SAMICoreDataType_MusicTagging_Result = 507,
    SAMICoreDataType_Chorus_Result = 508,
    SAMICoreDataType_MusicRetarget_Result = 509,
    SAMICoreDataType_Midi_Result = 510,
    SAMICoreDataType_Token_Result = 511,

} SAMICoreDataType;

typedef enum SAMI_CORE_DLL_EXPORT binaryType { binaryTypeFile, binaryTypeData } SAMICoreBinaryType;

typedef struct SAMI_CORE_DLL_EXPORT property {
    SAMICoreDataType type;
    SAMICorePropertyId id;
    int writable;
    void* data;
    unsigned int dataLen;
} SAMICoreProperty;

typedef struct SAMI_CORE_DLL_EXPORT properties {
    SAMICoreProperty* data;
    unsigned int dataLen;
} SAMICorePropertySet;

typedef struct SAMI_CORE_DLL_EXPORT audioBuffer {
    unsigned int numberChannels;
    unsigned int numberSamples;
    int isInterleave;
    float** data;
} SAMICoreAudioBuffer;

typedef struct SAMI_CORE_DLL_EXPORT audioBlock {
    SAMICoreDataType dataType;
    unsigned int numberAudioData;
    // abstract, can convert to any object such as midi, etc
    void* audioData;
} SAMICoreBlock;
/*****************************/

/***********Extractor*************/
// use for creating extractor
typedef struct SAMI_CORE_DLL_EXPORT extractorCreateParameter {
    unsigned int sampleRate;
    unsigned int numChannel;
} SAMICoreExtractorCreateParam;

//
typedef struct SAMI_CORE_DLL_EXPORT extractorParameter {
    char* name;
    char* strValue;
    float floatValue;
} SAMICoreExtractorParam;
/*****************************/

/***********Processor*************/
typedef struct SAMI_CORE_DLL_EXPORT audioProcessorParameter {
    int index;
    char name[128];
    float plainValue;
    float minPlainValue;
    float maxPlainValue;
    float scalingPower;
    int type;
    int scalingMode;
} SAMICoreProcessorParameter;
/*****************************/

/***********Processor*************/
typedef struct SAMI_CORE_DLL_EXPORT audioProcessorPrepareParameter {
    unsigned int sampleRate;
    unsigned int blockSize;
} SAMICoreProcessorPrepareParameter;

/*****************************/

typedef struct feature {
    float time;
    float duration;
    float* values;
    unsigned int numValues;
} SAMICoreFeature;

typedef struct featureArray {
    SAMICoreFeature* array;
    unsigned int numFeatures;
    unsigned int featureID;
} SAMICoreFeatureArray;

typedef struct featureSet {
    int numFeatureTypes;
    SAMICoreFeatureArray* set;
} SAMICoreFeatureSet;

/***********3A*************/
typedef struct SAMI_CORE_DLL_EXPORT audio3ACreateParameter {
    unsigned int sampleRate;
    unsigned int channels;
} SAMICore3ACreateParameter;

typedef struct SAMI_CORE_DLL_EXPORT audio3ASeekParameter {
    unsigned int pos;
    unsigned int mod;
} SAMICore3ASeekParameter;

typedef struct aecInput {
    SAMICoreAudioBuffer* ref;
    SAMICoreAudioBuffer* mic;
} SAMICoreAecInput;
/*****************************/

/*********time scaler**********/
typedef struct SAMI_CORE_DLL_EXPORT audioTimeScalerCreateParameter {
    unsigned int sampleRate;
    unsigned int channels;
} SAMICoreTimeScalerCreateParameter;
/*****************************/

/*********time align**********/
typedef SAMICoreAecInput SAMICoreTimeAlignParameter;
/*****************************/
/**
 * this struct used for creating context handle, @see SAMICoreIdentify_Processor_ContextWithString @see
 SAMICoreIdentify_Processor_ContextWithFileName
 * here is a small example:
 *
 * @code
 * const char* json_path = "/path/to/json/a.json";
 * SAMICoreContextCreateParameter create_param;
 * create_param.input = json;
 * create_param.resource_paths = NULL;
 * create_param.num_paths = 0;
 *
 * int ret = SAMICoreCreateHandleByIdentify(
 *              &handle, SAMICoreIdentify::SAMICoreIdentify_Processor_ContextWithFileName,
 *              &create_param);
 *
 * @endcode
 *
 * if there are resource paths that needs oto set, then you can write link this:
 * @code
 * SAMICoreContextCreateParameter create_param;
 * const char* json_path = "/path/to/json/a.json";
 * const char* path0 = {"./"};
 * const char* path1 = {"../"};
 * vector<const char*> paths{path0, path1};
 *
 * create_param.resource_paths = paths.data();
 * create_param.num_paths = 2;
 *
 * int ret = SAMICoreCreateHandleByIdentify(
                &handle, SAMICoreIdentify::SAMICoreIdentify_Processor_ContextWithFileName,
                (void *)(&create_param));
 *
 * @endcode
 */
typedef struct SAMI_CORE_DLL_EXPORT resourceParameter {
    /**
     * input json file path or json string, depends on which identity used
     *  1. SAMICoreIdentify_Processor_ContextWithStringWithResourcePaths, 'input' must be json string
     *  2. SAMICoreIdentify_Processor_ContextWithFileNameWithResourcePaths, 'input' must be json file path
     */
    const char* input;

    /**
     * number of resource search paths, can be 0 if there is no resource search paths
     */
    int numPaths;

    /**
     * resource search paths, is a string array.
     * the context use these paths to find resources it needed
     */
    const char** resourcePaths;
} SAMICoreResourceParameter;

/*********engine**********/
typedef struct SAMI_CORE_DLL_EXPORT engineCreateParameter {
    const char* resourcePath;
    int sampleRate;
} SAMICoreEngineCreateParameter;

typedef enum SAMI_CORE_DLL_EXPORT executorTaskType {
    TIME_ALIGN_MODEL = 0,
} SAMICoreExecutorTaskType;

typedef struct SAMI_CORE_DLL_EXPORT executorCreateParameter {
    void* engineHandle;
    int sampleRate;
    SAMICoreExecutorTaskType taskType;
    int outputNum;
} SAMICoreExecutorCreateParameter;

typedef struct SAMI_CORE_DLL_EXPORT executorContextCreateParameter {
    int sampleRate;
    int maxBlockSize;
    const char* modelBuffer;
    int modelLen;
    int numChannel;
} SAMICoreExecutorContextCreateParameter;

typedef struct SAMI_CORE_DLL_EXPORT binaryContextCreateParameter {
    SAMICoreBinaryType type;
    void* data;
    int dataLen;
} SAMICoreBinaryContextCreateParameter;

typedef struct SAMI_CORE_DLL_EXPORT parameterEvent {
    int portIndex;
    int parameterIndex;
    float normalizedValue;
} SAMICoreContextParameterEvent;

typedef struct SAMI_CORE_DLL_EXPORT jsonContextCreateParameter {
    int sampleRate;
    int maxBlockSize;
    const char* modelBuffer;
    int modelLen;
    int numChannel;
    const char* json_string;
} SAMICoreJSONContextCreateParameter;

typedef struct SAMI_CORE_DLL_EXPORT tuneToTargetCreateParameter {
    int sampleRate;
    const char* midiPath;
    float shift;
    float lag;
} SAMICoreTuneToTargetCreateParameter;
/***************** Server API ******************************/
typedef struct serverAPICommonParameter {
    const char* url;
    const char* appKey;
    const char* token;
    const char* format;
    const char* language;
    int sampleRate;
    int channel;
} SAMICoreServerAPICommonParameter;

typedef struct ttsContextCreateParameter {
    const char* url;
    const char* appKey;
    const char* token;
    const char* speaker;
    const char* format;
    int sampleRate;
    int speechRate;
    int pitchRate;
    bool enableTimestamp;
    int silenceDuration;
} SAMICoreTtsContextCreateParameter;

typedef struct ttsResult {
    float duration;
    char* phonemeAlignment;
    char* wordAlignment;
    char* data;
    int dataLength;
} SAMICoreTtsResult;

typedef struct asrContextCreateParameter {
    SAMICoreServerAPICommonParameter commonParameter;
    const char* language;
} SAMICoreAsrContextCreateParameter;

typedef struct asrResult {
    char* result;
} SAMICoreAsrResult;

typedef struct beatTrackingContextCreateParameter {
    SAMICoreServerAPICommonParameter commonParameter;
    bool groupResult;
} SAMICoreBTContextCreateParameter;

typedef struct beatTrackingResult {
    char* result;
} SAMICoreBeatTrackingResult;

typedef struct musicSourceSeparateContextParameter {
    SAMICoreServerAPICommonParameter commonParameter;
    const char* model;  // vocal/acc
    const char* outputFormat;
    int outputSampleRate;
    int outputChannel;
} SAMICoreMusicSourceSeparateContextParameter;

typedef struct musicSourceSeparateResult {
    char* data;
    int dataLength;
} SAMICoreMusicSourceSeparateResult;

typedef struct musicTaggingContextParameter {
    SAMICoreServerAPICommonParameter commonParameter;
    bool enableTag;
    int topNum;
    //const char* type;
    //const char** taskTypeArray;
    //int taskTypeSize;
} SAMICoreMusicTaggingContextParameter;

typedef struct musicTaggingResult {
    char* result;
} SAMICoreMusicTaggingResult;

typedef struct chorusContextParameter {
    SAMICoreServerAPICommonParameter commonParameter;
} SAMICoreChorusContextParameter;

typedef struct chorusResult {
    char* result;
} SAMICoreChorusResult;

typedef struct musicRetargetContextParameter {
    SAMICoreServerAPICommonParameter commonParameter;
    bool stamp;
    int targetSecs;
} SAMICoreMusicRetargetContextParameter;

typedef struct musicRetargetResult {
    char* result;
} SAMICoreMusicRetargetResult;

typedef struct midiContextParameter {
    SAMICoreServerAPICommonParameter commonParameter;
    bool midi;
} SAMICoreMidiContextParameter;

typedef struct midiResult {
    char* jsonResult;
    char* data;
    int dataLength;
} SAMICoreMidiResult;

/*****************************/

/************Error Code**************/
#define SAMI_OK 0

#define SAMI_BASE                         100000
#define SAMI_PARAM_ERROR                  (SAMI_BASE + 1)
#define SAMI_NOT_SUPPORT                  (SAMI_BASE + 2)
#define SAMI_NOT_IMPLEMENTATION           (SAMI_BASE + 3)
#define SAMI_JNI_PARSE_ERROR              (SAMI_BASE + 4)
#define SAMI_NOT_SUPPORTED_SAMPLE_RATE    (SAMI_BASE + 5)
#define SAMI_NOT_SUPPORTED_CHANNEL_LAYOUT (SAMI_BASE + 6)
#define SAMI_TOKEN_EXPIRED                (SAMI_BASE + 7)
#define SAMI_TOKEN_PARSE_FAILED           (SAMI_BASE + 8)
#define SAMI_TOKEN_APPKEY_NOT_MATCH       (SAMI_BASE + 9)
#define SAMI_CONTEXT_NOT_INIT             (SAMI_BASE + 10)
#define SAMI_SERVER_REQUEST_FAILED        (SAMI_BASE + 11)
#define SAMI_READ_FILE_FAILED             (SAMI_BASE + 12)
#define SAMI_NOT_ENOUGH_BUFFER_ERROR      (SAMI_BASE + 13)
#define SAMI_BLOCK_ERROR                  (SAMI_BASE + 14)

// processor
#define SAMI_PROCESSOR_BASE             (SAMI_BASE + 10000)
#define SAMI_CREATE_PROCESSOR_FAIL      (SAMI_PROCESSOR_BASE + 1)
#define SAMI_PROCESSOR_NOT_PREPARED     (SAMI_PROCESSOR_BASE + 2)
#define SAMI_PROCESSOR_IN_OUT_NOT_EQUAL (SAMI_PROCESSOR_BASE + 3)
#define SAMI_PROCESSOR_SET_STATE_FAIL   (SAMI_PROCESSOR_BASE + 4)

// extractor
#define SAMI_EXTRACTOR_BASE         (SAMI_BASE + 20000)
#define SAMI_CREATE_EXTRACTOR_FAIL  (SAMI_EXTRACTOR_BASE + 1)
#define SAMI_RELEASE_EXTRACTOR_FAIL (SAMI_EXTRACTOR_BASE + 2)

// music dsp context
#define SAMI_MSDP_CONTEXT_BASE                           (SAMI_BASE + 30000)
#define SAMI_MSDP_CONTEXT_NOT_PREPARED                   (SAMI_MSDP_CONTEXT_BASE + 1)
#define SAMI_MDSP_CONTEXT_INVALID_JSON                   (SAMI_MSDP_CONTEXT_BASE + 2)
#define SAMI_MDSP_CONTEXT_INVALID_PATH                   (SAMI_MSDP_CONTEXT_BASE + 3)
#define SAMI_MDSP_CONTEXT_INPUT_CONTEXT_IS_NULL          (SAMI_MSDP_CONTEXT_BASE + 4)
#define SAMI_MDSP_CONTEXT_EMPLACE_PARAMETER_EVENT_FAILED (SAMI_PROCESSOR_BASE + 4)

// 3a
#define SAMI_PROCESS_3A_BASE          (SAMI_BASE + 40000)
#define SAMI_PROCESS_RNNOISE_ERROR    (SAMI_PROCESS_3A_BASE + 1)
#define SAMI_PROCESS_TCNDENOISE_ERROR (SAMI_PROCESS_3A_BASE + 2)
#define SAMI_LOAD_AEC_MODEL_ERROR     (SAMI_PROCESS_3A_BASE + 3)
// sami engine
/*****************************/
#define SAMI_ENGINE_BASE                    (SAMI_BASE + 50000)
#define SAMI_ENGINE_CREATE_ERROR            (SAMI_ENGINE_BASE + 1)
#define SAMI_ENGINE_GETOUTPUT_ERROR         (SAMI_ENGINE_BASE + 2)
#define SAMI_ENGINE_FORWARD_ERROR           (SAMI_ENGINE_BASE + 3)
#define SAMI_ENGINE_START_EXECUTOR_ERROR    (SAMI_ENGINE_BASE + 4)
#define SAMI_ENGINE_UNSUPPORTED_BLOCK_ERROR (SAMI_ENGINE_BASE + 5)
#define SAMI_ENGINE_GETOUTPUT_NO_OUTPUT     (SAMI_ENGINE_BASE + 6)

//tob
#define SAMI_TOB_BASE      (SAMI_BASE + 60000)
#define SAMI_TOB_AUTH_FAIL (SAMI_TOB_BASE + 1)

SAMI_CORE_EXTERN_C_END
#endif  // SAMI_CORE_DEF_H
